@@ -1,7 +1,9 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 
-#include "config.h"
+#include <config.h>
 
 #include <dune/common/parallel/indexset.hh>
 #include <dune/common/parallel/plocalindex.hh>
@@ -12,16 +14,13 @@
 
 int main(int argc, char **argv)
 {
-  // This is a parallel programm so we need to
+  // This is a parallel program so we need to
   // initialize mpi first.
   Dune::MPIHelper& helper = Dune::MPIHelper::instance(argc, argv);
 
   // The rank of our process
   int rank = helper.rank();
 
-  // The type used as the local index
-  typedef Dune::ParallelLocalIndex<Flag> LocalIndex;
-
   // The type used as the global index
   typedef int GlobalIndex;
 
@@ -42,10 +41,10 @@ int main(int argc, char **argv)
 
   // Print the index set
   if(rank==0)
-    std::cout<<"Reordered lcoal indices:"<<std::endl;
+    std::cout<<"Reordered local indices:"<<std::endl;
 
   // Wait for all processes
-  helper.getCollectiveCommunication().barrier();
+  helper.getCommunication().barrier();
 
   std::cout<<indexSet<<std::endl;
   // Assign new local indices
