@@ -1,12 +1,13 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #include "config.h"
 
-#if HAVE_MPI
-
 #include <dune/common/parallel/indicessyncer.hh>
 #include <dune/common/sllist.hh>
 #include <string>
+#include <tuple>
 #include <iostream>
 
 enum GridFlags {
@@ -25,7 +26,7 @@ void deleteOverlapEntries(T& indices,
   typedef typename RemoteIndices::RemoteIndexList::const_iterator RemoteIterator;
   typedef Dune::SLList<std::pair<GlobalIndex,Attribute>, typename RemoteIndices::RemoteIndexList::Allocator> GlobalList;
   typedef typename GlobalList::ModifyIterator GlobalModifier;
-  typedef Dune::tuple<RemoteModifier,GlobalModifier,const RemoteIterator,const typename GlobalList::const_iterator,
+  typedef std::tuple<RemoteModifier,GlobalModifier,const RemoteIterator,const typename GlobalList::const_iterator,
       const GlobalList*, const typename RemoteIndices::RemoteIndexList*> IteratorTuple;
   typedef std::map<int,IteratorTuple> IteratorMap;
   typedef typename RemoteIndices::const_iterator RemoteMapIterator;
@@ -77,30 +78,30 @@ void deleteOverlapEntries(T& indices,
       for(iterator remote = iterators.begin(); remote != end; ++remote) {
 
         // Search for the index
-        while(Dune::get<0>(remote->second) != Dune::get<2>(remote->second)
-              && *(Dune::get<1>(remote->second)) < *index) {
+        while(std::get<0>(remote->second) != std::get<2>(remote->second)
+              && *(std::get<1>(remote->second)) < *index) {
           // increment all iterators
-          ++(Dune::get<0>(remote->second));
-          ++(Dune::get<1>(remote->second));
-          if(Dune::get<0>(remote->second)!=Dune::get<2>(remote->second))
-            assert(Dune::get<1>(remote->second)!=Dune::get<3>(remote->second));
+          ++(std::get<0>(remote->second));
+          ++(std::get<1>(remote->second));
+          if(std::get<0>(remote->second)!=std::get<2>(remote->second))
+            assert(std::get<1>(remote->second)!=std::get<3>(remote->second));
         }
 
         // Delete the entry if present
-        if(Dune::get<0>(remote->second) != Dune::get<2>(remote->second)) {
-          assert(Dune::get<1>(remote->second) != Dune::get<3>(remote->second));
+        if(std::get<0>(remote->second) != std::get<2>(remote->second)) {
+          assert(std::get<1>(remote->second) != std::get<3>(remote->second));
 
-          if(*(Dune::get<1>(remote->second)) == *index) {
+          if(*(std::get<1>(remote->second)) == *index) {
 
             std::cout<<rank<<": Deleting remote "<<
-            Dune::get<1>(remote->second)->first<<", "<<
-            Dune::get<1>(remote->second)->second<<" of process "
+            std::get<1>(remote->second)->first<<", "<<
+            std::get<1>(remote->second)->second<<" of process "
             << remote->first<<std::endl;
 
             // Delete entries
-            Dune::get<0>(remote->second).remove();
-            Dune::get<1>(remote->second).remove();
-            assert(Dune::get<4>(remote->second)->size()==Dune::get<5>(remote->second)->size());
+            std::get<0>(remote->second).remove();
+            std::get<1>(remote->second).remove();
+            assert(std::get<4>(remote->second)->size()==std::get<5>(remote->second)->size());
           }
         }
       }
@@ -334,24 +335,22 @@ public:
   int errorcode;
 };
 
-void MPI_err_handler(MPI_Comm *comm, int *err_code, ...){
-  DUNE_UNUSED_PARAMETER(comm);
+void MPI_err_handler([[maybe_unused]] MPI_Comm *comm, int *err_code, ...)
+{
   char *err_string=new char[MPI_MAX_ERROR_STRING];
   int err_length;
   MPI_Error_string(*err_code, err_string, &err_length);
   std::string s(err_string, err_length);
-  std::cerr << "An MPI Error ocurred:"<<std::endl<<s<<std::endl;
+  std::cerr << "An MPI Error occurred:"<<std::endl<<s<<std::endl;
   delete[] err_string;
   throw MPIError(s, *err_code);
 }
-#endif // HAVE_MPI
 
 int main(int argc, char** argv){
-#if HAVE_MPI
   MPI_Init(&argc, &argv);
   MPI_Errhandler handler;
-  MPI_Errhandler_create(MPI_err_handler, &handler);
-  MPI_Errhandler_set(MPI_COMM_WORLD, handler);
+  MPI_Comm_create_errhandler(MPI_err_handler, &handler);
+  MPI_Comm_set_errhandler(MPI_COMM_WORLD, handler);
   int procs, rank;
   MPI_Comm_size(MPI_COMM_WORLD, &procs);
   MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -362,7 +361,4 @@ int main(int argc, char** argv){
     MPI_Abort(MPI_COMM_WORLD, 1);
   MPI_Finalize();
   return 0;
-#else
-  return 77;
-#endif
 }
