@@ -1,6 +1,7 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
-// $Id$
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -188,7 +189,7 @@ int main(){
 
   if(0!=testRandomAccess()) {
     ret++;
-    cerr<< "Ransom Access failed!"<<endl;
+    cerr<< "Random Access failed!"<<endl;
   }
 
   if(0!=testSorting()) {
