@@ -1,9 +1,14 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifndef DUNE_FMatrix_WITH_CHECKING
+#define DUNE_FMatrix_WITH_CHECKING
+#endif
 
 #include <iostream>
 #include <algorithm>
@@ -11,7 +16,6 @@
 #include <dune/common/exceptions.hh>
 #include <dune/common/fvector.hh>
 #include <dune/common/diagonalmatrix.hh>
-#include <dune/common/unused.hh>
 
 #include "checkmatrixinterface.hh"
 
@@ -20,12 +24,16 @@ using namespace Dune;
 template<class K, int n>
 void test_matrix()
 {
-  typedef typename DiagonalMatrix<K,n>::size_type size_type DUNE_UNUSED;
+  [[maybe_unused]] typedef typename DiagonalMatrix<K,n>::size_type size_type;
 
   DiagonalMatrix<K,n> A(1);
   FieldVector<K,n> f;
   FieldVector<K,n> v;
 
+  // test constexpr size
+  static_assert(A.N() == n, "");
+  static_assert(A.M() == n, "");
+
   // assign matrix
   A=2;
 
@@ -51,10 +59,15 @@ void test_matrix()
   std::cout << f << std::endl;
 
   // assign to FieldMatrix
-  DUNE_UNUSED FieldMatrix<K,n,n> AFM = FieldMatrix<K,n,n>(A);
-  DUNE_UNUSED FieldMatrix<K,n,n> AFM2 = A;
-  DUNE_UNUSED FieldMatrix<K,n,n> AFM3;
+  [[maybe_unused]] FieldMatrix<K,n,n> AFM = FieldMatrix<K,n,n>(A);
+  [[maybe_unused]] FieldMatrix<K,n,n> AFM2 = A;
+  [[maybe_unused]] FieldMatrix<K,n,n> AFM3;
   AFM3 = A;
+
+  // test transposed
+  DiagonalMatrix<K,n> AT = A.transposed();
+  if (AT != A)
+    DUNE_THROW(FMatrixError, "Return value of DiagoalMatrix::transposed() incorrect!");
 }
 
 template<class K, int n>
@@ -70,7 +83,7 @@ void test_interface()
 
 void test_initialisation()
 {
-  DUNE_UNUSED Dune::DiagonalMatrix<int, 2> const b = { 1, 2 };
+  [[maybe_unused]] Dune::DiagonalMatrix<int, 2> const b = { 1, 2 };
 
   assert(b.diagonal(0) == 1);
   assert(b.diagonal(1) == 2);
