@@ -1,11 +1,15 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+#include <iostream>
+
 #include <dune/common/dynvector.hh>
 #include <dune/common/exceptions.hh>
-#include <iostream>
 
 using Dune::DynamicVector;
 
@@ -15,7 +19,20 @@ void dynamicVectorTest(int d) {
   DynamicVector<ct> v(d,1);
   DynamicVector<ct> w(d,2);
   DynamicVector<ct> z(d,2);
-  bool b DUNE_UNUSED;
+  [[maybe_unused]] bool b;
+
+  DynamicVector<ct> v2(v); // copy constructor
+  if (v!=v2)
+    DUNE_THROW(Dune::InvalidStateException,"Copy constructor does not work properly");
+
+  DynamicVector<ct> v3(std::move(v2)); // move constructor
+  if (v!=v3)
+    DUNE_THROW(Dune::InvalidStateException,"Move constructor does not work properly");
+
+  v2 = std::move(v3); // move assignment
+  if (v!=v2)
+    DUNE_THROW(Dune::InvalidStateException,"Move assignment does not work properly");
+
 
   // Test whether the norm methods compile
   (w+v).two_norm();
