@@ -1,17 +1,33 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <dune/common/fvector.hh>
-#include <dune/common/exceptions.hh>
-#include <dune/common/typetraits.hh>
-#include <dune/common/classname.hh>
-#include <dune/common/gmpfield.hh>
-#include <iostream>
+
 #include <complex>
+#include <iostream>
+#include <limits>
+#include <memory>
 #include <typeinfo>
-#include <cassert>
+#include <type_traits>
+
+#include <dune/common/classname.hh>
+#include <dune/common/exceptions.hh>
+#include <dune/common/fvector.hh>
+#include <dune/common/gmpfield.hh>
+#include <dune/common/quadmath.hh>
+#include <dune/common/typetraits.hh>
+
+struct FVectorTestException : Dune::Exception {};
+
+#define FVECTORTEST_ASSERT(EXPR)                            \
+  if(!(EXPR)) {                                             \
+    DUNE_THROW(FVectorTestException,                        \
+               "Test assertion " << #EXPR << " failed");    \
+  }                                                         \
+  static_assert(true, "enforce terminating ;")
 
 
 using Dune::FieldVector;
@@ -22,6 +38,17 @@ template<class ft, class rt, int d>
 struct FieldVectorMainTestCommons
 {
   FieldVectorMainTestCommons() {
+#if __GNUC__ != 5 || defined(__clang__)
+    static_assert(
+      !std::is_trivially_copyable<ft>::value || std::is_trivially_copyable< FieldVector<ft, d> >::value,
+      "FieldVector<T, ...> must be a trivially copyable type when T is a trivial type"
+      );
+#endif
+    static_assert(
+      std::is_standard_layout< FieldVector<ft, d> >::value,
+      "FieldVector<...> must be a standard layout type"
+      );
+
     ft a = 1;
     FieldVector<ft,d> v(1);
     FieldVector<ft,d> w(2);
@@ -29,29 +56,29 @@ struct FieldVectorMainTestCommons
     const FieldVector<ft,d> x(z);
     if (x.size()>0)
       a = x[0];
-    bool b DUNE_UNUSED;
-    rt n DUNE_UNUSED;
+    [[maybe_unused]] bool b;
+    [[maybe_unused]] rt n;
 
     std::cout << __func__ << "\t ( " << className(v) << " )" << std::endl;
 
     // test exported types
     static_assert(
-      Dune::is_same<ft,typename FieldVector<ft,d>::value_type>::value,
+      std::is_same<ft,typename FieldVector<ft,d>::value_type>::value,
       "FieldVector::value_type is not the correct type"
     );
 
     // test traits
     static_assert(
-      ( Dune::is_same< typename Dune::FieldTraits<
+      ( std::is_same< typename Dune::FieldTraits<
                 FieldVector<ft,d> >::field_type, ft >::value ),
       "FieldTraits<FieldVector> yields wrong field_type"
       );
     static_assert(
-      ( Dune::is_same< typename Dune::FieldTraits<ft>::real_type, rt >::value ),
+      ( std::is_same< typename Dune::FieldTraits<ft>::real_type, rt >::value ),
       "FieldTraits<field_type> yields wrong real_type"
       );
     static_assert(
-      ( Dune::is_same< typename Dune::FieldTraits<
+      ( std::is_same< typename Dune::FieldTraits<
                 FieldVector<ft,d> >::real_type, rt >::value ),
       "FieldTraits<FieldVector> yields wrong real_type"
       );
@@ -67,7 +94,7 @@ struct FieldVectorMainTestCommons
     // test op(vec,vec)
     z = v + w;
     z = v - w;
-    FieldVector<ft,d> z2 DUNE_UNUSED = v + w;
+    [[maybe_unused]] FieldVector<ft,d> z2 = v + w;
     w -= v;
     w += v;
 
@@ -76,6 +103,12 @@ struct FieldVectorMainTestCommons
     w -= a;
     w *= a;
     w /= a;
+    w = a * v;
+    w = v * a;
+    w = v / a;
+
+    // Negation
+    -v;
 
     // test scalar product, axpy
     a = v * w;
@@ -94,11 +127,18 @@ struct FieldVectorMainTestCommons
       v[i] = i;
     }
     s >> w;
-    assert(v == w);
+    FVECTORTEST_ASSERT(v == w);
 
     // test container methods
     typename FieldVector<ft,d>::size_type size = FieldVector<ft,d>::dimension;
-    assert(size == w.size());
+    FVECTORTEST_ASSERT(size == w.size());
+
+    if (w.size() > 0) {
+      FVECTORTEST_ASSERT(!w.empty());
+      FVECTORTEST_ASSERT(std::addressof(w[0]) == std::addressof(w.front()));
+      FVECTORTEST_ASSERT(std::addressof(w[0]) == w.data());
+      FVECTORTEST_ASSERT(std::addressof(w[d-1]) == std::addressof(w.back()));
+    }
   }
 };
 
@@ -119,7 +159,7 @@ struct FieldVectorMainTest
     // assignment to vector of complex
     FieldVector< std::complex<rt> ,d> cv = v;
     cv = a;
-    const FieldVector< std::complex<rt> ,d> ccv DUNE_UNUSED = x;
+    [[maybe_unused]] const FieldVector< std::complex<rt> ,d> ccv = x;
   }
 };
 
@@ -142,7 +182,7 @@ struct ScalarOperatorTest
     testft c = 2;
     FieldVector<ft,1> v(2);
     FieldVector<ft,1> w(2);
-    bool b DUNE_UNUSED;
+    [[maybe_unused]] bool b;
 
     std::cout << __func__ << "\t ( " << className(v) << " )" << std::endl;
 
@@ -179,9 +219,9 @@ struct ScalarOperatorTest
     v += 1; // make sure v!=0
     v = a / v;
 
-    v -= v;
+    v -= w;
     v -= a;
-    v += v;
+    v += w;
     v += a;
     v *= a;
     a += 1; // make sure a!=0
@@ -205,7 +245,7 @@ struct ScalarOrderingTest
     ft c = 2;
     FieldVector<ft,1> v(2);
     FieldVector<ft,1> w(2);
-    bool b DUNE_UNUSED;
+    [[maybe_unused]] bool b;
 
     std::cout << __func__ << "\t ( " << className(v) << " )" << std::endl;
 
@@ -255,10 +295,10 @@ struct DotProductTest
 {
   DotProductTest() {
     typedef std::complex<rt> ct;
-    DUNE_UNUSED const rt myEps = Epsilon<rt>::value();
+    [[maybe_unused]] const rt myEps = Epsilon<rt>::value();
 
     static_assert(
-      ( Dune::is_same< typename Dune::FieldTraits<rt>::real_type, rt>::value ),
+      ( std::is_same< typename Dune::FieldTraits<rt>::real_type, rt>::value ),
       "DotProductTest requires real data type as template parameter!"
       );
 
@@ -268,8 +308,8 @@ struct DotProductTest
 
     std::cout << __func__ << "\t \t ( " << Dune::className(one) << " and " << Dune::className(iVec) << ")" << std::endl;
 
-    const bool isRealOne = Dune::is_same<typename Dune::FieldTraits<rt>::field_type,typename Dune::FieldTraits<rt>::real_type>::value;
-    const bool isRealIVec = Dune::is_same<typename Dune::FieldTraits<ct>::field_type,typename Dune::FieldTraits<ct>::real_type> ::value;
+    const bool isRealOne = std::is_same<typename Dune::FieldTraits<rt>::field_type,typename Dune::FieldTraits<rt>::real_type>::value;
+    const bool isRealIVec = std::is_same<typename Dune::FieldTraits<ct>::field_type,typename Dune::FieldTraits<ct>::real_type> ::value;
     static_assert(isRealOne,"1-vector expected to be real");
     static_assert(!isRealIVec,"i-vector expected to be complex");
 
@@ -279,41 +319,41 @@ struct DotProductTest
 
     // one^H*one should equal d
     result = dot(one,one);
-    assert(std::abs(result-length)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result-length)<= myEps);
     result = one.dot(one);
-    assert(std::abs(result-length)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result-length)<= myEps);
 
 
     // iVec^H*iVec should equal d
     result = dot(iVec,iVec);
-    assert(std::abs(result-length)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result-length)<= myEps);
     result = iVec.dot(iVec);
-    assert(std::abs(result-length)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result-length)<= myEps);
 
 
     // test that we do conjugate first argument
     result = dot(one,iVec);
-    assert(std::abs(result-length*I)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result-length*I)<= myEps);
     result = dot(one,iVec);
-    assert(std::abs(result-length*I)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result-length*I)<= myEps);
 
 
     // test that we do not conjugate second argument
     result = dot(iVec,one);
-    assert(std::abs(result+length*I)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result+length*I)<= myEps);
     result = iVec.dot(one);
-    assert(std::abs(result+length*I)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result+length*I)<= myEps);
 
 
     // test that dotT does not conjugate at all
     result = dotT(one,one) + one*one;
-    assert(std::abs(result-ct(2)*length)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result-ct(2)*length)<= myEps);
     result = dotT(iVec,iVec) + iVec*iVec;
-    assert(std::abs(result+ct(2)*length)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result+ct(2)*length)<= myEps);
     result = dotT(one,iVec) + one*iVec;
-    assert(std::abs(result-ct(2)*length*I)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result-ct(2)*length*I)<= myEps);
     result = dotT(iVec,one) + iVec*one;
-    assert(std::abs(result-ct(2)*length*I)<= myEps);
+    FVECTORTEST_ASSERT(std::abs(result-ct(2)*length*I)<= myEps);
 
   }
 };
@@ -323,10 +363,10 @@ template <class rt, int d>
 struct DotProductTest<rt, d, false>
 {
   DotProductTest() {
-    DUNE_UNUSED const rt myEps = Epsilon<rt>::value();
+    [[maybe_unused]] const rt myEps = Epsilon<rt>::value();
 
     static_assert(
-      ( Dune::is_same< typename Dune::FieldTraits<rt>::real_type, rt>::value ),
+      ( std::is_same< typename Dune::FieldTraits<rt>::real_type, rt>::value ),
       "DotProductTest requires real data type as template parameter!"
       );
 
@@ -334,7 +374,7 @@ struct DotProductTest<rt, d, false>
 
     std::cout << __func__ << "\t \t ( " << Dune::className(one) << " only)" << std::endl;
 
-    const bool isRealOne = Dune::is_same<typename Dune::FieldTraits<rt>::field_type,typename Dune::FieldTraits<rt>::real_type>::value;
+    const bool isRealOne = std::is_same<typename Dune::FieldTraits<rt>::field_type,typename Dune::FieldTraits<rt>::real_type>::value;
     static_assert(isRealOne,"1-vector expected to be real");
 
     rt result = rt();
@@ -342,13 +382,13 @@ struct DotProductTest<rt, d, false>
 
     // one^H*one should equal d
     result = dot(one,one);
-    assert(abs(result-length)<= myEps);
+    FVECTORTEST_ASSERT(abs(result-length)<= myEps);
     result = one.dot(one);
-    assert(abs(result-length)<= myEps);
+    FVECTORTEST_ASSERT(abs(result-length)<= myEps);
 
     // test that dotT does not conjugate at all
     result = dotT(one,one) + one*one;
-    assert(abs(result-rt(2)*length)<= myEps);
+    FVECTORTEST_ASSERT(abs(result-rt(2)*length)<= myEps);
   }
 };
 
@@ -425,24 +465,46 @@ public:
   }
 };
 
-// Make sure that a vector with only NaN entries has norm NaN.
-// Prior to r6914, the infinity_norm would be zero; see also FS #1147.
+template <class V>
+void checkNormNAN(V const &v, int line) {
+  if (!std::isnan(v.one_norm())) {
+    std::cerr << "error: norm not NaN: one_norm() on line "
+              << line << " (type: " << Dune::className(v[0]) << ")"
+              << std::endl;
+    std::exit(-1);
+  }
+  if (!std::isnan(v.two_norm())) {
+    std::cerr << "error: norm not NaN: two_norm() on line "
+              << line << " (type: " << Dune::className(v[0]) << ")"
+              << std::endl;
+    std::exit(-1);
+  }
+  if (!std::isnan(v.infinity_norm())) {
+    std::cerr << "error: norm not NaN: infinity_norm() on line "
+              << line << " (type: " << Dune::className(v[0]) << ")"
+              << std::endl;
+    std::exit(-1);
+  }
+}
+
+// Make sure that vectors with NaN entries have norm NaN.
+// See also bug flyspray/FS#1147
+template <typename T>
 void
-test_nan()
+test_nan(T const &mynan)
 {
-  double mynan = 0.0/0.0;
-
-  Dune::FieldVector<double, 2> v2(mynan);
-  assert(std::isnan(v2.infinity_norm()));
-  assert(std::isnan(v2.one_norm()));
-  assert(std::isnan(v2.two_norm()));
-  assert(std::isnan(v2.two_norm2()));
-
-  Dune::FieldVector<double, 0> v0(mynan);
-  assert(0.0 == v0.infinity_norm());
-  assert(0.0 == v0.one_norm());
-  assert(0.0 == v0.two_norm());
-  assert(0.0 == v0.two_norm2());
+  {
+    Dune::FieldVector<T, 2> v = { mynan, mynan };
+    checkNormNAN(v, __LINE__);
+  }
+  {
+    Dune::FieldVector<T, 2> v = { mynan, 0 };
+    checkNormNAN(v, __LINE__);
+  }
+  {
+    Dune::FieldVector<T, 2> v = { 0, mynan };
+    checkNormNAN(v, __LINE__);
+  }
 }
 
 void
@@ -454,46 +516,116 @@ test_infinity_norms()
   Dune::FieldVector<std::complex<double>, 2> v;
   v[0] = threefour;
   v[1] = eightsix;
-  assert(std::abs(v.infinity_norm()     -10.0) < 1e-10); // max(5,10)
-  assert(std::abs(v.infinity_norm_real()-14.0) < 1e-10); // max(7,14)
+  FVECTORTEST_ASSERT(std::abs(v.infinity_norm()     -10.0) < 1e-10); // max(5,10)
+  FVECTORTEST_ASSERT(std::abs(v.infinity_norm_real()-14.0) < 1e-10); // max(7,14)
 }
 
 void
 test_initialisation()
 {
-  DUNE_UNUSED Dune::FieldVector<int, 2> const b = { 1, 2 };
+  [[maybe_unused]] Dune::FieldVector<int, 2> const b = { 1, 2 };
 
-  assert(b[0] == 1);
-  assert(b[1] == 2);
+  FVECTORTEST_ASSERT(b[0] == 1);
+  FVECTORTEST_ASSERT(b[1] == 2);
 }
 
+void fieldvectorMathclassifiersTest() {
+  double nan = std::nan("");
+  double inf = std::numeric_limits<double>::infinity();
+
+  FieldVector<double,3> fv_normal(1.);
+  FieldVector<double,3> fv_nan(1.);
+  FieldVector<double,3> fv_inf(1.);
+
+  fv_nan[2] = nan;
+  fv_inf[2] = inf;
+
+  //test vector containing only doubles
+  if(Dune::isNaN(fv_normal) == true) {
+    std::abort();
+  }
+  if(Dune::isInf(fv_normal) == true) {
+     std::abort();
+  }
+  if(Dune::isFinite(fv_normal) == false) {
+    std::abort();
+  }
+
+  //test vector containing a NaN-entry
+  if(Dune::isNaN(fv_nan) == false) {
+     std::abort();
+  }
+  if(Dune::isInf(fv_nan) == true) {
+     std::abort();
+  }
+  if(Dune::isFinite(fv_nan) == true) {
+    std::abort();
+  }
+
+  //test vector containing an infinity-entry
+  if(Dune::isNaN(fv_inf) == true) {
+    std::abort();
+  }
+  if(Dune::isInf(fv_inf) == false) {
+    std::abort();
+  }
+  if(Dune::isFinite(fv_inf) == true) {
+    std::abort();
+  }
+}
+
+
 int main()
 {
-  try {
+  {
+    FieldVectorTest<int, 1>();
+    FieldVectorTest<float, 1>();
+    FieldVectorTest<double, 1>();
+    FieldVectorTest<long double, 1>();
     FieldVectorTest<int, 3>();
     FieldVectorTest<float, 3>();
     FieldVectorTest<double, 3>();
     FieldVectorTest<long double, 3>();
 #if HAVE_GMP
-    // we skip the complex test and the int test, as these will be very hard to implement with GMPField
-    typedef Dune::GMPField<128u> ft;
-    FieldVectorMainTest<ft,ft,3>();
-    FieldVectorMainTest<ft,ft,2>();
-    FieldVectorMainTest<ft,ft,1>();
-    FieldVectorMainTest<ft,ft,0>();
-    ScalarOperatorTest<ft>();
-    ScalarOrderingTest<ft>();
-    DotProductTest<ft,3>();
+    {
+      // we skip the complex test and the int test, as these will be very hard to implement with GMPField
+      typedef Dune::GMPField<128u> ft;
+      FieldVectorMainTest<ft,ft,3>();
+      FieldVectorMainTest<ft,ft,2>();
+      FieldVectorMainTest<ft,ft,1>();
+      FieldVectorMainTest<ft,ft,0>();
+      ScalarOperatorTest<ft>();
+      ScalarOrderingTest<ft>();
+      DotProductTest<ft,3>();
+    }
 #endif // HAVE_GMP
 
-    test_nan();
+#if HAVE_QUADMATH
+    {
+      // we skip the int test, as these will be very hard to implement with Float128
+      typedef Dune::Float128 ft;
+      FieldVectorMainTest<ft,ft,3>();
+      FieldVectorMainTest<ft,ft,2>();
+      FieldVectorMainTest<ft,ft,1>();
+      FieldVectorMainTest<ft,ft,0>();
+      ScalarOperatorTest<ft>();
+      ScalarOrderingTest<ft>();
+      DotProductTest<ft,3>();
+    }
+#endif
+
+    //test the mathclassifiers Dune::isNaN, Dune::isInf, Dune::isFinite
+    fieldvectorMathclassifiersTest();
+
+    {
+      double nan = std::nan("");
+      test_nan(nan);
+    }
+    {
+      std::complex<double> nan( std::nan(""), 17 );
+      test_nan(nan);
+    }
     test_infinity_norms();
     test_initialisation();
-  } catch (Dune::Exception& e) {
-    std::cerr << e << std::endl;
-    return 1;
-  } catch (...) {
-    std::cerr << "Generic exception!" << std::endl;
-    return 2;
   }
 }
