@@ -1,5 +1,7 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #include <config.h>
 
 #include <dune/common/parallel/mpihelper.hh>
@@ -27,7 +29,7 @@ int main(int argc, char** argv)
               << e.what() << std::endl;
   }
 
-  mpihelper.getCollectiveCommunication().barrier();
+  mpihelper.getCommunication().barrier();
   if (mpihelper.rank() == 0)
     std::cout << "---- guard(MPI_COMM_WORLD)" << std::endl;
   try
@@ -50,7 +52,7 @@ int main(int argc, char** argv)
               << e.what() << std::endl;
   }
 
-  mpihelper.getCollectiveCommunication().barrier();
+  mpihelper.getCommunication().barrier();
   if (mpihelper.rank() == 0)
     std::cout << "---- guard(MPIHelper)" << std::endl;
   try
@@ -70,7 +72,7 @@ int main(int argc, char** argv)
   }
 
 
-  mpihelper.getCollectiveCommunication().barrier();
+  mpihelper.getCommunication().barrier();
   if (mpihelper.rank() == 0)
     std::cout << "---- manual error" << std::endl;
   try
@@ -87,7 +89,7 @@ int main(int argc, char** argv)
               << e.what() << std::endl;
   }
 
-  mpihelper.getCollectiveCommunication().barrier();
+  mpihelper.getCommunication().barrier();
   if (mpihelper.rank() == 0)
     std::cout << "---- done" << std::endl;
 }
