@@ -1,12 +1,15 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <dune/common/parallel/mpihelper.hh>
 #include <iostream>
 
+#include <dune/common/parallel/mpihelper.hh>
+
 int main(int argc, char** argv)
 {
 
@@ -21,14 +24,14 @@ int main(int argc, char** argv)
   {
     Helper& mpi = Helper::instance(argc, argv);
 
-    Helper::MPICommunicator comm DUNE_UNUSED = mpi.getCommunicator();
+    [[maybe_unused]] Helper::MPICommunicator comm = mpi.getCommunicator();
     comm= mpi.getCommunicator();
   }
 
   {
     Helper& mpi = Helper::instance(argc, argv);
 
-    Helper::MPICommunicator comm DUNE_UNUSED = mpi.getCommunicator();
+    [[maybe_unused]] Helper::MPICommunicator comm = mpi.getCommunicator();
     comm= mpi.getCommunicator();
 
 #ifdef MPIHELPER_PREINITIALIZE
