@@ -1,11 +1,15 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <cstdint>
+#include <vector>
+
 #include <dune/common/poolallocator.hh>
-#include <dune/common/alignment.hh>
 #include <dune/common/fmatrix.hh>
 
 using namespace Dune;
@@ -33,25 +37,25 @@ struct testPoolMain
     //int chunkSize   = Pool<T,size>::chunkSize;
     //int alignedSize = Pool<T,size>::alignedSize;
 
-    unsigned long* oelements = new unsigned long[10*elements];
+    std::vector<std::uintptr_t> oelements(10*elements);
 
     typedef typename Pool<T,size>::Chunk Chunk;
 
     //Fill 10 chunks
     for(int chunk=0; chunk < 10; ++chunk) {
       //std::cout<< std::endl<<"Chunk "<<chunk<<" ";
-      unsigned long element = reinterpret_cast<unsigned long>(pool.allocate());
+      std::uintptr_t element = reinterpret_cast<std::uintptr_t>(pool.allocate());
       //void* celement = reinterpret_cast<void*>(element);
       //std::cout << element<<" "<< celement<<",  "<<std::endl;
 
       Chunk* currentChunk = pool.chunks_;
 
-      assert(element==reinterpret_cast<unsigned long>(currentChunk->memory_));
-      unsigned long end = reinterpret_cast<unsigned long>(currentChunk->chunk_)+Pool<T,size>::chunkSize;
+      assert(element==reinterpret_cast<std::uintptr_t>(currentChunk->chunk_));
+      std::uintptr_t end = reinterpret_cast<std::uintptr_t>(currentChunk->chunk_)+Pool<T,size>::chunkSize;
 
-      if(element< reinterpret_cast<unsigned long>(currentChunk->chunk_))
+      if(element< reinterpret_cast<std::uintptr_t>(currentChunk->chunk_))
       {
-        std::cerr <<" buffer overflow during first alloc: "<<reinterpret_cast<unsigned long>(currentChunk->chunk_)
+        std::cerr <<" buffer overflow during first alloc: "<<reinterpret_cast<std::uintptr_t>(currentChunk->chunk_)
                   <<">"<<element<<"+"<<sizeof(T)<<std::endl;
         return ++ret;
       }
@@ -65,12 +69,12 @@ struct testPoolMain
 
       for(int i=1; i < elements; i++)
       {
-        element = reinterpret_cast<unsigned long>(pool.allocate());
+        element = reinterpret_cast<std::uintptr_t>(pool.allocate());
         //celement = reinterpret_cast<void*>(element);
         //std::cout << element<<" "<<celement<<",  "<<std::endl;
 
-        if(element< reinterpret_cast<unsigned long>(currentChunk->chunk_)) {
-          std::cerr <<" buffer underflow during first alloc: "<<reinterpret_cast<unsigned long>(currentChunk->chunk_)
+        if(element< reinterpret_cast<std::uintptr_t>(currentChunk->chunk_)) {
+          std::cerr <<" buffer underflow during first alloc: "<<reinterpret_cast<std::uintptr_t>(currentChunk->chunk_)
                     <<">"<<element<<"+"<<sizeof(T)<<std::endl;
           return ++ret;
         }
@@ -94,7 +98,6 @@ struct testPoolMain
 
     for(int i=0; i < elements*10; ++i)
       pool.free(reinterpret_cast<void*>(oelements[i]));
-    delete[] oelements;
 
     return ret;
   }
@@ -108,7 +111,7 @@ int testPool()
   int ret=0;
 
   std::cout<<"Checking "<<typeid(T).name()<<" sizeof="<<sizeof(T)<<" with size "<< size<<
-  " alignment="<<AlignmentOf<T>::value<<std::endl;
+  " alignment="<< alignof(T) <<std::endl;
 
   ret += testPoolMain<0,T>::test();
   ret += testPoolMain<size,T>::test();
@@ -134,7 +137,7 @@ int testPoolAllocator()
     std::cerr<<"ERROR: deallocation should not work with copied allocators."<<std::endl;
 #endif
   }
-  catch(std::bad_alloc)
+  catch(const std::bad_alloc&)
   {}
   pool1.allocate(1);
   double *d1=pool2.allocate(1);
@@ -157,7 +160,7 @@ int main(int, char **)
 
   ret+=testPoolAllocator();
 
-  std::cout<<AlignmentOf<UnAligned>::value<<" "<<sizeof(UnAligned)<<std::endl;
+  std::cout<< alignof(UnAligned) <<" "<<sizeof(UnAligned)<<std::endl;
 
   ret += testPool<UnAligned>();
 
