@@ -1,11 +1,11 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#warning sllist is known to fail with some optimization flags (FS493)
-
 #include <dune/common/sllist.hh>
 #include <dune/common/test/iteratortest.hh>
 #include <dune/common/poolallocator.hh>
@@ -82,7 +82,7 @@ int check(const Dune::SLList<T,A>& alist, const T* vals)
   int i=0;
   for(iterator iter = alist.begin(); iter != alist.end(); ++iter, i++) {
     if( vals[i] != *iter ) {
-      std::cerr<<" List missmatch! "<<__FILE__<<":"<<__LINE__<<std::endl;
+      std::cerr<<" List mismatch! "<<__FILE__<<":"<<__LINE__<<std::endl;
       return 1;
     }
   }
@@ -141,7 +141,7 @@ int testAssign()
   List::iterator biter=blist.begin(), aiter=alist.begin();
   for(; aiter!=alist.end(); ++aiter, ++biter)
     if(*aiter!=*biter) {
-      std::cerr<<"Asignment failed "<<__FILE__<<":"<<__LINE__<<std::endl;
+      std::cerr<<"Assignment failed "<<__FILE__<<":"<<__LINE__<<std::endl;
       return 1;
     }
   return 0;
