@@ -1,5 +1,7 @@
 // -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 // vi: set et ts=4 sw=2 sts=2:
+// SPDX-FileCopyrightInfo: Copyright (C) DUNE Project contributors, see file LICENSE.md in module root
+// SPDX-License-Identifier: LicenseRef-GPL-2.0-only-with-DUNE-exception
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -8,6 +10,7 @@
 #define DEBUG_NEW_DELETE 3
 
 #include <dune/common/debugallocator.hh>
+#if HAVE_MPROTECT
 
 #include <iostream>
 #include <csignal>
@@ -88,14 +91,26 @@ void new_delete_tests()
   z4 = 0;
 }
 
+#endif // HAVE_MPROTECT
+
 int main(int, char**)
 {
 #if EXPECTED_SIGNAL
   std::signal(EXPECTED_SIGNAL, std::_Exit);
 #endif
+#if EXPECTED_ALT_SIGNAL
+  std::signal(EXPECTED_ALT_SIGNAL, std::_Exit);
+#endif
 
+#if HAVE_MPROTECT
   basic_tests();
   allocator_tests();
   new_delete_tests();
+#endif
+
+#ifdef EXPECTED_SIGNAL
+  return 1;
+#else
   return 0;
+#endif
 }
